//+------------------------------------------------------------------+
//|                                               LondonBreakout.mq4 |
//+------------------------------------------------------------------+

/*+------------------------------------------------------------------+
 *
 * Version history:
 *
 * V9.1: [Squalou]
 *     - "monday-morning fix": box wouldn't draw on mondays when the box time includes midnight
 *       (local time 00:00 is between StartTime and EndTime).
 *
 * V9: [Squalou]
 *     - indicator Input parameters can now be dynamically modified by changing the input values
 *       in the system GlobalVariables(F3);
 *       StartTime, EndTime and SessionEndTime can be entered as hhmm integer format (eg 700 for 07:00)
 *       The Indicator will automatically redraw all boxes/levels when it detects the changes;
 *       GlobalVariables can also be changed using the "London Breakout Input Calculator" script.
 *
 * V8.1: [Squalou]
 *     - indicator now sets up a new set of GlobalVaraibles to "export" the settings,
 *       for use in other scripts or EAs;
 *
 * V8: [Squalou] the "swiss-army knife" version!!!
 *     - added "MinExtentInPips"(0) and "MaxExtentInPips"(0) inputs:
 *       to set the min/max box extent limits for computing the entry/TP/SL prices;
 *       0(default) means no min/max limitation (as before);
 *       Allows to keep SL/TP levels into "reasonable" limits on too-high or too-low pre-breakout volatility days;
 *       Tip: setting "MinExtentInPips" and "MaxExtentInPips" to the same non-0 value
 *       will result in constant SL/TP arround the box median price.
 *       Does not interfere with the Blue/Red "NO TRADE" drawing.
 *     - fixed the "Lopuch glitch window" issue...
 *       the box now forms only when the last bar is fully completed.
 *       Note that the last bar of the box INCLUDES the bar that STARTS at the EndTime.
 *       It means that, in M15 chart, with EndTime="07:00",
 *       the box includes the bar [07:00-07:15[, and is formed at 07:15 and not before.
 *     - now you can use the "London Breakout scriptV2.mq4" script on different charts for the same pair,
 *       setup with different Indicator inputs, eg 03:00-06:00 on one chart, and 04:00-07:00 on another chart;
 *       However, you need to change the "MagicNumber" input in the script if you want it
 *       to place different orders than the ones that were already placed by the script on that pair on other charts.
 *       V7 was overwriting the GlobalVariables output at EVERY TICK.
 *
 * V7: [Squalou]
 *     - added "showProfitZone"(true) and "ProfitColor"(LightGreen) inputs:
 *       draws buy/sell Profit zones in LightGreen during Session time
 *     - Buy/Sell Entry/TP/SL prices are now "output" to system GlobalVariables,
 *       for use with the "London Breakout scriptV2.mq4" script;
 *     - added "objPrefix"("LB-") input: prefix to be applied to all drawn objects;
 *     
 * V6.1: [Squalou]
 *     - fixed TP levels computation when TPFactor - EntryFactor != 1; (e.g. EntryFactor=0.3 and TPFactor=1.5)
 *       Doesn't show up with default settings (0.3,1.3);
 *       Affects V4,V5,V6;
 *
 * V6: [Squalou]
 *     - added "LevelsResizeFactor"(1.0) input: applies a factor to all computed levels;
 *       allows to adjust the default SL levels for instance.
 *       Example: with EntryFactor=7.5, TPFactor=17.5, and LevelsResizeFactor=0.1,
 *       this will keep the SAME Entry and TP prices as the defaults (0.3,1.3,1.0),
 *       but it will squeeze the buy/sell SL prices to only 1/10th of the box size.
 *       The general formula to keep the same XXprice for a given XXfactor is:
 *       new_XXfactor = ((0.5 + old_XXfactor) / LevelsResizeFactor) - 0.5;
 *       (e.g: new_Entryfactor = ((0.5 + 0.3) / 0.1) - 0.5 = 7.5;
 *             new_TPfactor    = ((0.5 + 1.3) / 0.1) - 0.5 = 17.5;)
 *
 * V5: [Squalou]
 *     - added "SessionColor"(PapayaWhip) input highlight the Session periods with a different "background"
 *
 * V4: [Squalou]
 *     - added "MaxBoxSizeInPips"(50), "BoxColorOK"(LightBlue), "BoxColorNOK"(Red):
 *       shows breakout boxes in different colors to distinguish "trade/no-trade" cases:
 *       the box turns Red when its height exceeds "MaxBoxSizeInPips" pips;
 *     - show Fib TP/SL levels in pips in addition to Price values;
 *
 * V3.1: [Sangmane]
 *     - added "MaxBoxSizeInPips"(50): do not display the breakout box
 *       when its height exceeds "MaxBoxSizeInPips" pips, i.e. on "no-trade" days.
 *
 * V1: original version from mer071898 at forexfactory.com
 *     Read full thread here: http://www.forexfactory.com/showthread.php?t=230640
 *
 *+------------------------------------------------------------------+
 */

#property link      "http://www.forexfactory.com/showthread.php?t=230640"
#property copyright "London Breakout"

#property indicator_chart_window
#property indicator_buffers 6
#property indicator_color1 CLR_NONE
#property indicator_color2 CLR_NONE
#property indicator_color3 CLR_NONE
#property indicator_color4 CLR_NONE
#property indicator_color5 CLR_NONE
#property indicator_color6 CLR_NONE

double buy_buffer[];
double sell_buffer[];
double buy_SL_buffer[];
double sell_SL_buffer[];
double buy_TP_buffer[];
double sell_TP_buffer[];

#define VERSION "V9.1 for EA"

extern string Info         = "London Breakout";
extern string StartTime    = "09:00";    // time for start of price establishment window
extern string EndTime      = "09:59";    // time for end of price establishment window
extern string SessionEndTime= "14:00";   // end of daily session; tomorrow is another day!
extern int    MaxBoxSizeInPips = 50;     // box size limit to show Red "NO TRADE" box
extern double EntryFactor  = 0.3;
extern double TPFactor     = 1.3;
extern double LevelsResizeFactor = 1.0;
extern int    MinExtentInPips = 0; // actual box extent min limit for computing trade levels;
extern int    MaxExtentInPips = 0; // actual box extent max limit for computing trade levels; 0 will default to MaxBoxSizeInPips;
extern color  BoxColorOK   = LightBlue;
extern color  BoxColorNOK  = Red;
extern color  LevelColor   = Black;
extern color  SessionColor = PapayaWhip; // show Session periods with a different background color
extern int    NumDays      = 60;         // days back
extern int    FibLength    = 9;
extern bool   showProfitZone = true;
extern color  ProfitColor  = LightGreen;

extern string objPrefix    = "LB-";  // all objects drawn by this indicator will be prefixed with this


//--------------------------------------------------------

// GLOBAL variables

double pip;
int digits;
int BarsBack;

datetime tBoxStart;
datetime tBoxEnd;
datetime tSessionStart;
datetime tSessionEnd;

double vSellStop;
double vBuyStop;

double LongPendingEntryPrice;
double ShortPendingEntryPrice;
double LongTP;
double ShortTP;

int StartShift;
int EndShift;
datetime tLastBoxDrawn;
double boxHigh;
double boxLow;
double boxExtent;
double boxMedianPrice;

string symbol;

int iStartTime,iEndTime,iSessionEndTime;

//+------------------------------------------------------------------+
int init()  {
//+------------------------------------------------------------------+

  SetIndexBuffer(0,buy_buffer);
  SetIndexBuffer(1,sell_buffer);
  SetIndexBuffer(2,buy_SL_buffer);
  SetIndexBuffer(3,sell_SL_buffer);
  SetIndexBuffer(4,buy_TP_buffer);
  SetIndexBuffer(5,sell_TP_buffer);

  symbol = Symbol();
  
  string s="London Breakout Indicator levels drawn from ["+StartTime+"-"+EndTime+"] ends trading at "+SessionEndTime+" maximum box size "+DoubleToStr(MaxBoxSizeInPips,0)+" pips";
  if (LevelsResizeFactor!=1) s = s+"(x"+DoubleToStr(LevelsResizeFactor,1)+")";
  if (MinExtentInPips!=0) s = s+" min:"+DoubleToStr(MinExtentInPips,0)+"p";
  if (MaxExtentInPips!=0) s = s+" max:"+DoubleToStr(MaxExtentInPips,0)+"p";
  Comment(s);

  RemoveObjects(objPrefix);	
  getpip();

  // save SETTINGS into system global variables, to make them available for other scripts/EAs
  // Note: GlobalVariableSet() only allow<s double(or int) variables, not strings... so times are converted into integer values
  iStartTime = time_string_to_int(StartTime);
  GlobalVariableSet(objPrefix+symbol+"-StartTime", iStartTime);
  iEndTime = time_string_to_int(EndTime);
  GlobalVariableSet(objPrefix+symbol+"-EndTime", iEndTime);
  iSessionEndTime = time_string_to_int(SessionEndTime);
  GlobalVariableSet(objPrefix+symbol+"-SessionEndTime", iSessionEndTime);
  GlobalVariableSet(objPrefix+symbol+"-MaxBoxSizeInPips", MaxBoxSizeInPips);
  GlobalVariableSet(objPrefix+symbol+"-EntryFactor", EntryFactor);
  GlobalVariableSet(objPrefix+symbol+"-TPFactor", TPFactor);
  GlobalVariableSet(objPrefix+symbol+"-LevelsResizeFactor", LevelsResizeFactor);
  GlobalVariableSet(objPrefix+symbol+"-MinExtentInPips", MinExtentInPips);
  GlobalVariableSet(objPrefix+symbol+"-MaxExtentInPips", MaxExtentInPips);

  BarsBack = NumDays*(PERIOD_D1/Period());
  tLastBoxDrawn = 0;

  return(0);
}/*init*/

//+------------------------------------------------------------------+
int deinit()  {
//+------------------------------------------------------------------+

  RemoveObjects(objPrefix);	

  return(0);
}/*deinit*/


//+------------------------------------------------------------------+
void start()
//+------------------------------------------------------------------+
{

  handle_bars (Bars-IndicatorCounted()-1);

}//start()

//+------------------------------------------------------------------+
void handle_bars(int bars)
//+------------------------------------------------------------------+
{
  int i, limit;

  limit = MathMin(BarsBack,bars);

  for (i=limit; i>=0; i--) {
    handle_single_bar(i);

  }

  return(0);
}//handle_bars()

//+------------------------------------------------------------------+
void handle_single_bar(int i) // i = bar number: 0=current(last) bar
//+------------------------------------------------------------------+
{
    int StartShift;
    int EndShift;

    datetime thistime = Time[i];

    /* Import SETTINGS input values from GlobalVariables:
     * this allows to CHANGE the INPUTS settings "on-the-fly"
     * by simply changing the values in the GlobalVariables,
     * without having to run the Indicator again.
     * SETTINGS GlobalVariables can be changed manually (F3)
     * or using the "London Breakout Input Calculator" script;
     */
    if (import_LB_Indi_SETTINGS(objPrefix,symbol) == true) {
      // indicator settings have been changed:
      // force redrawing the Box and Playground
      tLastBoxDrawn = 0;
      RemoveObjects(objPrefix);
      Alert (objPrefix+symbol+": INPUTS CHANGED !!!");
      Print (objPrefix+symbol+"-StartTime = "+StartTime);
      Print (objPrefix+symbol+"-EndTime = "+EndTime);
      Print (objPrefix+symbol+"-SessionEndTime = "+SessionEndTime);
      Print (objPrefix+symbol+"-MaxBoxSizeInPips = "+MaxBoxSizeInPips);
      Print (objPrefix+symbol+"-EntryFactor = "+EntryFactor);
      Print (objPrefix+symbol+"-TPFactor = "+TPFactor);
      Print (objPrefix+symbol+"-LevelsResizeFactor = "+LevelsResizeFactor);
      Print (objPrefix+symbol+"-MinExtentInPips = "+MinExtentInPips);
      Print (objPrefix+symbol+"-MaxExtentInPips = "+MaxExtentInPips);
      // redraw completely all boxes for this indicator
      handle_bars (Bars);
      return;
    }

    compute_LB_Indi_LEVELS(thistime);

    // get TP and SL in pips
    double LongTPpips      = (LongTP-LongPendingEntryPrice)/pip;
    double ShortTPpips     = (ShortPendingEntryPrice-ShortTP)/pip;
    double BuyStoppips     = (vBuyStop-LongPendingEntryPrice)/pip;
    double SellStoppips    = (ShortPendingEntryPrice-vSellStop)/pip;
    
    if (boxExtent <= MaxBoxSizeInPips * pip) {
    buy_buffer[i] = LongPendingEntryPrice;
    sell_buffer[i] = ShortPendingEntryPrice;
    buy_SL_buffer[i] = vBuyStop;
    sell_SL_buffer[i] = vSellStop;
    buy_TP_buffer[i] = LongTP;
    sell_TP_buffer[i] = ShortTP;
    /*
    Print("buy_buffer: ",buy_buffer[i]);
    Print("sell_buffer: ",sell_buffer[i]);
    Print("buy_SL_buffer: ",buy_SL_buffer[i]);
    Print("sell_SL_buffer: ",sell_SL_buffer[i]);
    Print("buy_TP_buffer: ",buy_TP_buffer[i]);
    Print("sell_TP_buffer: ",sell_TP_buffer[i]);
    */
    } else {
      buy_buffer[i] = 0;
      sell_buffer[i] = 0;
      buy_SL_buffer[i] = 0;
      sell_SL_buffer[i] = 0;
      buy_TP_buffer[i] = 0;
      sell_TP_buffer[i] = 0;
    }

    // show session period with a different "background" color
    drawBoxOnce (objPrefix+"Session-"+TimeToStr(tSessionStart,TIME_DATE),tSessionStart,0,tSessionEnd,LongTP*2,SessionColor,1, STYLE_SOLID, true);

    // draw session results at session end time
    if (thistime == tSessionEnd) drawSessionResults(tBoxEnd, tSessionEnd, LongPendingEntryPrice, LongTP, vBuyStop, ShortPendingEntryPrice, ShortTP, vSellStop);

    // draw pre-breakout box blue/red once per Session:
    if (tLastBoxDrawn != tBoxEnd && thistime >= tBoxEnd + Period()*60) {
      tLastBoxDrawn = tBoxEnd; // won't redraw until next box

      // draw pre-breakout box blue/red:
      string boxName = objPrefix+"Box-"+TimeToStr(Time[i],TIME_DATE)+"-"+StartTime+"-"+EndTime;
      if (boxExtent <= MaxBoxSizeInPips * pip) {
        drawBox (boxName,tBoxStart,boxLow,tBoxEnd,boxHigh,BoxColorOK,1, STYLE_SOLID, true);
        DrawLbl(objPrefix+"Lbl-"+TimeToStr(Time[i],TIME_DATE)+"-"+StartTime+"-"+EndTime, DoubleToStr(boxExtent/pip,0)+"p"/*+"[+"+DoubleToStr(LongTPpips,0)+",-"+DoubleToStr(BuyStoppips,0)+"]"*/, tBoxStart+(tBoxEnd-tBoxStart)/2,boxLow, 12, "Arial Black", LevelColor, 3);
      } else { // "NO TRADE" box
        drawBox (boxName,tBoxStart,boxLow,tBoxEnd,boxHigh,BoxColorNOK,1, STYLE_SOLID, true);
        DrawLbl(objPrefix+"Lbl-"+TimeToStr(Time[i],TIME_DATE)+"-"+StartTime+"-"+EndTime, "NO TRADE ("+DoubleToStr(boxExtent/pip,0)+"p)", tBoxStart+(tBoxEnd-tBoxStart)/2,boxLow, 12, "Arial Black", BoxColorNOK, 3);
      }

      // draw profit/loss boxes for the session
      if (showProfitZone) {
        drawBox (objPrefix+"BuyProfitZone-" +TimeToStr(tSessionStart,TIME_DATE),tSessionStart,LongPendingEntryPrice,tSessionEnd,LongTP,ProfitColor,1, STYLE_SOLID, true);
        drawBox (objPrefix+"SellProfitZone-"+TimeToStr(tSessionStart,TIME_DATE),tSessionStart,ShortPendingEntryPrice,tSessionEnd,ShortTP,ProfitColor,1, STYLE_SOLID, true);
      }

      // save price levels into system global variables, to make them available for other scripts
      GlobalVariableSet(objPrefix+symbol+"-LongEntry", LongPendingEntryPrice);
      GlobalVariableSet(objPrefix+symbol+"-ShortEntry", ShortPendingEntryPrice);
      GlobalVariableSet(objPrefix+symbol+"-LongTP", LongTP);
      GlobalVariableSet(objPrefix+symbol+"-ShortTP", ShortTP);
      GlobalVariableSet(objPrefix+symbol+"-LongSL", vBuyStop);
      GlobalVariableSet(objPrefix+symbol+"-ShortSL", vSellStop);

      // draw "fib" lines for entry+stop+TP levels:
      string objname = objPrefix+"Fibo-" + TimeToStr(tBoxEnd,TIME_DATE | TIME_SECONDS);
      ObjectCreate(objname,OBJ_FIBO,0,tBoxStart,vBuyStop,tBoxStart+FibLength*60*10,vSellStop);
      ObjectSet(objname,OBJPROP_RAY,false);
      ObjectSet(objname,OBJPROP_LEVELCOLOR,LevelColor); 
      ObjectSet(objname,OBJPROP_FIBOLEVELS,6);
      ObjectSet(objname,OBJPROP_LEVELSTYLE,STYLE_SOLID);
      ObjectSet(objname,OBJPROP_FIRSTLEVEL+0,0.0);
      ObjectSet(objname,OBJPROP_FIRSTLEVEL+1,1.0);
      ObjectSet(objname,OBJPROP_FIRSTLEVEL+2,-EntryFactor);
      ObjectSet(objname,OBJPROP_FIRSTLEVEL+3,-TPFactor);
      ObjectSet(objname,OBJPROP_FIRSTLEVEL+4,1+EntryFactor);
      ObjectSet(objname,OBJPROP_FIRSTLEVEL+5,1+TPFactor);
      ObjectSetFiboDescription(objname,0,"Stop Loss for SHORT = %$  ("+DoubleToStr(SellStoppips,0)+"p)");
      ObjectSetFiboDescription(objname,1,"Stop Loss for LONG = %$  ("+DoubleToStr(BuyStoppips,0)+"p)");
      ObjectSetFiboDescription(objname,2,"Enter LONG = %$ ("+LongPendingEntryPrice+")");
      ObjectSetFiboDescription(objname,3,"Take Profit for LONG = %$  (+"+DoubleToStr(LongTPpips,0)+"p)");
      ObjectSetFiboDescription(objname,4,"Enter SHORT = %$ ("+ShortPendingEntryPrice+")");
      ObjectSetFiboDescription(objname,5,"Take Profit for SHORT = %$  (+"+DoubleToStr(ShortTPpips,0)+"p)");
    }
}//handle_single_bar(i)
  

//+------------------------------------------------------------------+
bool import_LB_Indi_SETTINGS(string objPrefix, string symbol)
//+------------------------------------------------------------------+
// returns true when settings have been changed
{
  bool ret=false;

  int _iStartTime            = GlobalVariableGet(objPrefix+symbol+"-StartTime");
  int _iEndTime              = GlobalVariableGet(objPrefix+symbol+"-EndTime");
  int _iSessionEndTime       = GlobalVariableGet(objPrefix+symbol+"-SessionEndTime");
  double _MaxBoxSizeInPips   = GlobalVariableGet(objPrefix+symbol+"-MaxBoxSizeInPips");
  double _EntryFactor        = GlobalVariableGet(objPrefix+symbol+"-EntryFactor");
  double _TPFactor           = GlobalVariableGet(objPrefix+symbol+"-TPFactor");
  double _LevelsResizeFactor = GlobalVariableGet(objPrefix+symbol+"-LevelsResizeFactor");
  double _MinExtentInPips    = GlobalVariableGet(objPrefix+symbol+"-MinExtentInPips");
  double _MaxExtentInPips    = GlobalVariableGet(objPrefix+symbol+"-MaxExtentInPips");

  // check if Inputs have been changed from previous tick

  if (_iStartTime != iStartTime) {
    iStartTime = _iStartTime;
    StartTime  = time_int_to_string(iStartTime);
    ret = true;
  }
  if (_iEndTime != iEndTime) {
    iEndTime = _iEndTime;
    EndTime  = time_int_to_string(iEndTime);
    ret = true;
  }
  if (_iSessionEndTime != iSessionEndTime) {
    iSessionEndTime = _iSessionEndTime;
    SessionEndTime  = time_int_to_string(iSessionEndTime);
    ret = true;
  }
  if (_MaxBoxSizeInPips != 0 && _MaxBoxSizeInPips != MaxBoxSizeInPips) {
    MaxBoxSizeInPips = _MaxBoxSizeInPips;
    ret = true;
  }
  if (_EntryFactor != 0 && _EntryFactor != EntryFactor) {
    EntryFactor = _EntryFactor;
    ret = true;
  }
  if (_TPFactor != 0 && _TPFactor != TPFactor) {
    TPFactor = _TPFactor;
    ret = true;
  }
  if (_LevelsResizeFactor != 0 && _LevelsResizeFactor != LevelsResizeFactor) {
    LevelsResizeFactor = _LevelsResizeFactor;
    ret = true;
  }
  if (_MinExtentInPips != 0 && _MinExtentInPips != MinExtentInPips) {
    MinExtentInPips = _MinExtentInPips;
    ret = true;
  }
  if (_MaxExtentInPips != 0 && _MaxExtentInPips != MaxExtentInPips) {
    MaxExtentInPips = _MaxExtentInPips;
    ret = true;
  }

  return (ret);
}

//+------------------------------------------------------------------+
void compute_LB_Indi_LEVELS(datetime thistime)
//+------------------------------------------------------------------+
{
    tBoxStart = StrToTime(TimeToStr(thistime,TIME_DATE) + " "  + StartTime);
    tBoxEnd   = StrToTime(TimeToStr(thistime,TIME_DATE) + " "  + EndTime);
    if (tBoxStart > tBoxEnd) tBoxStart = tBoxStart - 24*60*60; // midnight wrap fix
    if (thistime < tBoxEnd) { // consider the last PAST box
      tBoxStart = tBoxStart - 24*60*60;
      tBoxEnd   = tBoxEnd   - 24*60*60;
    }

    tSessionStart = tBoxEnd;
    tSessionEnd = StrToTime(TimeToStr(tSessionStart,TIME_DATE) + " "  + SessionEndTime);
    if (tSessionStart > tSessionEnd) tSessionEnd = tSessionEnd + 24*60*60; // midnight wrap fix

    if (TimeDayOfWeek(tBoxStart)==0) return; //don't show saturday; sunday=0~saturday=6;

    //get hi/lo
    StartShift = iBarShift(NULL,0,tBoxStart);
    EndShift   = iBarShift(NULL,0,tBoxEnd);
    boxHigh = High[iHighest(NULL,0,MODE_HIGH,(StartShift-EndShift+1),EndShift)];
    boxLow  = Low[iLowest(NULL,0,MODE_LOW,(StartShift-EndShift+1),EndShift)];
    boxExtent       = boxHigh - boxLow;
    boxMedianPrice = (boxHigh+boxLow)/2;

    // "resize" the Box with LevelsResizeFactor before computing levels
    double extent = boxExtent;
    if (MaxExtentInPips > 0 && boxExtent > MaxExtentInPips*pip) extent = MaxExtentInPips*pip;
    if (MinExtentInPips > 0 && boxExtent < MinExtentInPips*pip) extent = MinExtentInPips*pip;
    extent = extent * LevelsResizeFactor;
    vSellStop = boxMedianPrice + extent/2;
    vBuyStop  = boxMedianPrice - extent/2;

    // Entry and TP prices
    LongPendingEntryPrice  = NormalizeDouble(vSellStop + extent*EntryFactor, Digits);
    ShortPendingEntryPrice = NormalizeDouble(vBuyStop  - extent*EntryFactor, Digits);
    LongTP  = NormalizeDouble(vSellStop + extent*TPFactor, Digits);
    ShortTP = NormalizeDouble(vBuyStop  - extent*TPFactor, Digits);
}

//--------------------------------------------------------------------------------------
int time_string_to_int(string ts)
//--------------------------------------------------------------------------------------
// convert a time string "hh:mm" into an integer hhmm
{
  int t;
  t = StrToTime("1970.01.01 " + ts)/60;
  return (MathRound(t/60)*100+(t%60));
}

//--------------------------------------------------------------------------------------
string time_int_to_string(int t)
//--------------------------------------------------------------------------------------
// convert a time integer hhmm into a string "hh:mm"
{
  if (t>9999) return (StringSubstr(TimeToStr(t),11,5)); // time value includes DATE -- compatible with Indicator < V9
  return ((t/100)+":"+(t%100)); // time is in hhmm integer format (e.g 1400 for "14:00")
}


//--------------------------------------------------------------------------------------
void drawSessionResults (
//--------------------------------------------------------------------------------------
  datetime tSessionStart, datetime ttSessionEnd,
  double LongEntry,  double LongTP,  double LongSL,
  double ShortEntry, double ShortTP, double ShortSL
)
{

// TODO: compute session's winners/loosers and draw green/red corresponding boxes

}

//--------------------------------------------------------------------------------------
// getpip
//--------------------------------------------------------------------------------------

void getpip()
{
   if(Digits==2 || Digits==4) pip = Point;
   else if(Digits==3 || Digits==5) pip = 10*Point;
   else if(Digits==6) pip = 100*Point;
      
	if (Digits == 3 || Digits == 2) digits = 2;
	else digits = 4;
} /* getpip*/


//--------------------------------------------------------------------------------------
// RemoveObjects
//--------------------------------------------------------------------------------------

void RemoveObjects(string Pref)
{   
   int i;
   string objname = "";

   for (i = ObjectsTotal(); i >= 0; i--) {
      objname = ObjectName(i);
      if (StringFind(objname, Pref, 0) > -1) ObjectDelete(objname);
   }
} /* RemoveObjects*/


//--------------------------------------------------------------------------------------
// drawBox
//--------------------------------------------------------------------------------------

void drawBox (
  string objname,
  datetime tStart, double vStart, 
  datetime tEnd,   double vEnd,
  color c, int width, int style, bool bg
)
{
  if (ObjectFind(objname) == -1) {
    ObjectCreate(objname, OBJ_RECTANGLE, 0, tStart,vStart,tEnd,vEnd);
  } else {
    ObjectSet(objname, OBJPROP_TIME1, tStart);
    ObjectSet(objname, OBJPROP_TIME2, tEnd);
    ObjectSet(objname, OBJPROP_PRICE1, vStart);
    ObjectSet(objname, OBJPROP_PRICE2, vEnd);
  }

  ObjectSet(objname,OBJPROP_COLOR, c);
  ObjectSet(objname, OBJPROP_BACK, bg);
  ObjectSet(objname, OBJPROP_WIDTH, width);
  ObjectSet(objname, OBJPROP_STYLE, style);
} /* drawBox */


//--------------------------------------------------------------------------------------
// drawBoxOnce: draw a Box only once; if it already exists, do nothing
//--------------------------------------------------------------------------------------

void drawBoxOnce (
  string objname,
  datetime tStart, double vStart, 
  datetime tEnd,   double vEnd,
  color c, int width, int style, bool bg
)
{
  if (ObjectFind(objname) != -1) return;
  
  ObjectCreate(objname, OBJ_RECTANGLE, 0, tStart,vStart,tEnd,vEnd);
  ObjectSet(objname,OBJPROP_COLOR, c);
  ObjectSet(objname, OBJPROP_BACK, bg);
  ObjectSet(objname, OBJPROP_WIDTH, width);
  ObjectSet(objname, OBJPROP_STYLE, style);
} /* drawBoxOnce */

//--------------------------------------------------------------------------------------
// drawFixedLbl
//--------------------------------------------------------------------------------------

/*
void drawFixedLbl(string objname, string s, int Corner, int DX, int DY, int FSize, string Font, color c, bool bg)
{
   if (ObjectFind(objname) < 0) ObjectCreate(objname, OBJ_LABEL, 0, 0, 0);
   
   ObjectSet(objname, OBJPROP_CORNER, Corner);
   ObjectSet(objname, OBJPROP_XDISTANCE, DX);
   ObjectSet(objname, OBJPROP_YDISTANCE, DY);
   ObjectSet(objname,OBJPROP_BACK, bg);      
   ObjectSetText(objname, s, FSize, Font, c);
} // drawFixedLbl
*/

//--------------------------------------------------------------------------------------
// DrawLbl
//--------------------------------------------------------------------------------------

void DrawLbl(string objname, string s, int LTime, double LPrice, int FSize, string Font, color c, int width)
{
  if (ObjectFind(objname) < 0) {
    ObjectCreate(objname, OBJ_TEXT, 0, LTime, LPrice);
  } else {
    if (ObjectType(objname) == OBJ_TEXT) {
      ObjectSet(objname, OBJPROP_TIME1, LTime);
      ObjectSet(objname, OBJPROP_PRICE1, LPrice);
    }
  }

  ObjectSet(objname, OBJPROP_FONTSIZE, FSize);
  ObjectSetText(objname, s, FSize, Font, c);
} /* DrawLbl*/

WindowRedraw();


//end

