//+------------------------------------------------------------------+
//| sqDynamicBreakoutBox.mq4
//| by Squalou, 2011.07.05
//+------------------------------------------------------------------+

#property copyright "by Squalou, 2011.07.05"
#property link      "http://www.forexfactory.com/showthread.php?t=302007"


/*+------------------------------------------------------------------+
 * sqDynamicBreakoutBox.mq4, by Squalou, 2011.07.05:
 * - draws Dynamic Breakout Boxes based on consolidation areas;
 * - consolidation areas are defined by a minimum period of time during which
 *   the Range of PA remained below a minimum Range value;
 * - Inputs:
 *   - BoxLength and BoxTimeFrame: define the minimum duration of the congestion areas
 *     Better than specifying a number of bars, because this will not independ
 *     on the chart's timeframe;
 *   - BoxRange: the Box Range in pips;
 *
 * This indicator was inspired by a thread opened by "forexhard" on ForexFactory.
 * http://www.forexfactory.com/showthread.php?t=302007
 * Thank you forexhard for this!
 *
//+------------------------------------------------------------------+
*/


#property indicator_chart_window

#property indicator_buffers 5

#property indicator_color1 Yellow // Breakout Box High Line
#property indicator_width1 3
#property indicator_color2 Yellow // Breakout Box Low Line
#property indicator_width2 3

#property indicator_color3 Magenta // Breakout Box High Line Continuation
#property indicator_color4 Magenta // Breakout Box Low Line Continuation


//+------------------------------------------------------------------+
//---- input parameters
extern int    BoxLength           = 4;
extern int    BoxTimeFrame        = 60;
extern int    BoxRange     = 30;
extern color  BreakoutBoxColor    = DodgerBlue;

//+------------------------------------------------------------------+


//---- buffers
double box_high[],box_low[];
double box_high_cont[],box_low_cont[];
double periodForMinRange[];

int _period,_minP,_maxP,_prevP;
double _Range;

string prefix;

double pip;
int pipMult,pipMultTab[]={1,10,1,10,1,10,100}; // multiplier to convert pips to Points;

//+------------------------------------------------------------------+
int init()
//+------------------------------------------------------------------+
{
  pipMult = pipMultTab[Digits];
  pip = Point * pipMult;

  prefix = WindowExpertName()+BoxLength+" "+BoxTimeFrame+" "+BoxRange;
  
  _Range = BoxRange * pip; // convert pips to price range

  if (BoxTimeFrame==0) BoxTimeFrame=Period();
  _period = BoxLength * BoxTimeFrame/Period();
  _minP   = _period;
  _prevP  = _period;

  IndicatorShortName(WindowExpertName()+"("+_period+")");

  IndicatorBuffers(5);
  int b=-1;
  b++;SetIndexBuffer(b,box_high); SetIndexLabel(b,"Breakout Box High");
  b++;SetIndexBuffer(b,box_low);  SetIndexLabel(b,"Breakout Box Low");
  b++;SetIndexBuffer(b,box_high_cont); SetIndexLabel(b,"Breakout Box High Continued");
  b++;SetIndexBuffer(b,box_low_cont);  SetIndexLabel(b,"Breakout Box Low Continued");
  b++;SetIndexBuffer(b,periodForMinRange); SetIndexLabel(b,"periodForMinRange");

  ArrayInitialize(box_high,EMPTY_VALUE);
  ArrayInitialize(box_low,EMPTY_VALUE);
  ArrayInitialize(box_high_cont,EMPTY_VALUE);
  ArrayInitialize(box_low_cont,EMPTY_VALUE);

  return(0);
}

int deinit()
//+------------------------------------------------------------------+
{
  RemoveObjects(prefix);
}


//+------------------------------------------------------------------+
int start()
//+------------------------------------------------------------------+
{
  double upper,lower;

  for (int i = Bars-IndicatorCounted(); i >= 0; i--)
  {
    // continue the previous Box high/low lines
    box_high_cont[i] = box_high_cont[i+1];
    box_low_cont[i]  = box_low_cont[i+1];

    // get the period over which PA Range remained below BoxRange
    _period = GetPeriodForMinRange(i, _minP, 999, _Range, _prevP);
    _prevP = _period;
    periodForMinRange[i] = _period;
    upper = High[iHighest(NULL, 0, MODE_HIGH, _period, i)];
    lower = Low [iLowest (NULL, 0, MODE_LOW,  _period, i)];

    //draw a Dynamic Breakout Box when Price Range remains below BoxRange during more than BoxLength
    if (_period > _minP) { // Price range remained below BoxRange longer than BoxLength:
      // draw a new Breakout Box, or extend the Box if one is already drawing
      if (box_high[i+1]==EMPTY_VALUE) { // no Box was forming:
        // start a new Box ONLY if a the previous Box is older than BoxLength
        if (box_high[i+_minP] == EMPTY_VALUE) {
          // limit the Box Range to BoxRange;
          // snap the Box to the extreme that is opposite of the first(oldest) candle of the Box
          if (High[i+_period-1]==upper) upper = lower+_Range;
          if (Low[i+_period-1]==lower)  lower = upper-_Range;
          // draw High and Low Yellow lines starting from the beginning of the Box
          for (int jj = i+_minP; jj>=i; jj--) {
            box_high[jj] = upper;
            box_low[jj]  = lower;
            //box_high_cont[jj] = upper;
            //box_low_cont[jj]  = lower;
          }
          // make the "Box Continuation lines" look discontinued
          box_high_cont[i+1] = EMPTY_VALUE;
          box_low_cont[i+1]  = EMPTY_VALUE;
          box_high_cont[i] = upper;
          box_low_cont[i]  = lower;
          //box_high_cont[i+_minP] = EMPTY_VALUE;
          //box_low_cont[i+_minP]  = EMPTY_VALUE;
          // draw a Box where it started to form:
          drawBox(prefix+TimeToStr(Time[i]), Time[i+_minP], upper, Time[i], lower, BreakoutBoxColor, 0, STYLE_SOLID, true, "");
        }
      } else { // a Box was already forming:
        // extend it ONLY IF price did not break out already
        if (Low[i] >= box_low[i+1] && High[i] <= box_high[i+1]) {
          // price is still inside the Box limits: extend the forming Box
          box_high[i] = box_high[i+1];
          box_low[i]  = box_low[i+1];
        } else {
          // Price has broken out of the forming Box: stop the forming Box
          box_high[i] = EMPTY_VALUE;
          box_low[i]  = EMPTY_VALUE;
        }
      }
    }

  }
  return(0);
}

//+------------------------------------------------------------------+
int GetPeriodForMinRange(int shift, int MinP, int MaxP, double MinRange, int prevP)
//+------------------------------------------------------------------+
{
  int P;
  double range;
  // calc P so that the Price Range (=Hi-Lo) remains in a "reasonably large value" over the P
  P=prevP; // start with previous P
  if (_get_range(P,shift) >= MinRange) {//range is OK for this P value: try shorter P values
    for (; P>=MinP; P--) {
      if (_get_range(P,shift) < MinRange) return(P+1);//previous P value was the limit
    }
    return(P);
  }
  //try higher P values
  for (P=prevP+1; P<MaxP; P++) {
    if (_get_range(P,shift) >= MinRange) return(P);
  }
  return(MaxP);
}

  double _get_range(int period, int shift)
  {
    return(High[iHighest(NULL,0,MODE_HIGH,period,shift)] - Low[iLowest(NULL,0,MODE_LOW,period,shift)]);
  }

//--------------------------------------------------------------------------------------
void drawBox (string objname, datetime tStart, double vStart, datetime tEnd, double vEnd, color c, int width, int style, bool bg, string comment)
//--------------------------------------------------------------------------------------
{
  if (ObjectFind(objname) == -1) {
    ObjectCreate(objname, OBJ_RECTANGLE, 0, tStart,vStart,tEnd,vEnd);
  } else {
    ObjectSet(objname, OBJPROP_TIME1, tStart);
    ObjectSet(objname, OBJPROP_TIME2, tEnd);
    ObjectSet(objname, OBJPROP_PRICE1, vStart);
    ObjectSet(objname, OBJPROP_PRICE2, vEnd);
  }

  ObjectSet(objname,OBJPROP_COLOR, c);
  ObjectSet(objname, OBJPROP_BACK, bg);
  ObjectSet(objname, OBJPROP_WIDTH, width);
  ObjectSet(objname, OBJPROP_STYLE, style);
  //ObjectSetText(objname, comment);
} /* drawBox */

//--------------------------------------------------------------------------------------
void RemoveObjects(string prefix)
//--------------------------------------------------------------------------------------
{   
   int i;
   string objname;

   for (i = ObjectsTotal(); i >= 0; i--) {
      objname = ObjectName(i);
      if (StringFind(objname, prefix, 0) > -1) ObjectDelete(objname);
   }
} /* RemoveObjects*/

//+------------------------------------------------------------------+

